// op_winamp.cpp

class CControl
{
public:
	int Image(connection * conn,int type);
	int TopFrame(connection * conn);
	int SmallTitle(connection * conn);
	int Title(connection * conn);
	int Shutdown(connection * conn);
	int Clear(connection * conn);
	int Download(connection * conn);
	int Load(connection * conn);
	int List(connection * conn);
	int Next(connection * conn);
	int Prev(connection * conn);
	int Play(connection * conn);
	int Stop(connection * conn);
	int StopSlow(connection * conn);
	int Pause(connection * conn);
	int Volume(connection * conn);
	int Playmode(connection * conn);
	int About(connection * conn);
	int Main(connection * conn);
	int WebAdmin(connection * conn);
	int ApplyAdmin(connection * conn);
	int pda(connection * conn);
	int Delete(connection * conn);
	int User(connection * conn);
	int SetUser(connection * conn);
	int UserStyle(connection * conn);
	int Sort(connection * conn);
	int SavePlaylist(connection * conn);
	
};
